 /* Author: Manjunath M Prabhu
  * The cmds class uses the moteIF interface to generate the command packets and send it to the
  * mote connected to the guage
  *
  * The base station uses the TOSBase application provided by TinyOS
  */
package net.tinyos.tools;

import net.tinyos.util.*;
import net.tinyos.message.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class cmds extends Panel implements MessageListener {
    static Properties p = new Properties();
    public static final byte READ_ADC = 1;
    public static final byte SET_PWR = 2;
    public static final byte PRG_PGA = 3;
    public static final byte SET_GAIN = 4;
    public static final byte SET_OFFSET = 5;
    public static final byte READ_REG = 6;
    public static final byte READ_GAIN = 7;
    public static final byte READ_OFFSET = 8;
    public static final byte READ_TEMP = 9;
    public static final byte MOTE_RST = 10;
    public static final byte TIMER_STOP = 11;
    public static boolean CALIBRATION = false;
    public static boolean SPAN_CALIBRATION = false;
    public static final double EPSILON=0.005;
    
    public static MoteIF mote = new MoteIF((Messenger)null);
    public static double gain_entry; 
    public static double offset_entry;
    public static String logfile;
    public static int counter=0;
    public static final double vZero=1.1;
    public static double setOffset=1.1;
    public static double total_adc,average_adc,total_uE,average_uE;
    public boolean read_log_done = false; 
    public int src_id;
    
    public static final double r1 = 5000;
	public static final double rG = 1000;
	public static final double gageFactor = 2.105;
	public static final double vExcitation = 2.066;
    
    public static FileOutputStream out, gainOut;
    public static PrintStream ps, ps1; 
    public static BufferedReader in=new BufferedReader(new InputStreamReader(System.in));    
    public static BufferedReader inputFile;
    
    public static Date date=new Date();
    
    newPanel display;
        
    public static final short TOS_BCAST_ADDR = (short) 0xffff;
	//public static final short TOS_UART_ADDR = (short) 0xffff;
    
    cmds(newPanel display) {
    	this.display=display;
		mote.registerListener(new Volt_Msg(), this);	
    }
    
    
    public static void usage() {
	System.err.println("Usage: java net.tinyos.tools.cmd_line"+
			   " <command> [arguments]");
	System.err.println("\twhere <command> and [arguments] can be one of the following:");
	System.err.println("\t\t read adc and the timer repeat value");
	System.err.println("\t\t set power and value to be set(0-31)");
	//System.err.println("\t\t read rssi");
	//System.err.println("\t\t read temp");
	}

    public static byte restoreSequenceNo() {
	try {
	    FileInputStream fis = new FileInputStream("bcast.properties");
	    p.load(fis);
	    byte i = (byte)Integer.parseInt(p.getProperty("sequenceNo", "1"));
	    fis.close();
	    return i;
		} catch (IOException e) {
	    p.setProperty("sequenceNo", "1");
	    return 1;
		}
    }
    public static void saveSequenceNo(int i) {
	try {
	    FileOutputStream fos = new FileOutputStream("bcast.properties");
	    p.setProperty("sequenceNo", Integer.toString(i));
	    p.store(fos, "#Properties for BcastInject\n");
	} catch (IOException e) {
	    System.err.println("Exception while saving sequence number" +
			       e);
	    e.printStackTrace();
		}
    }

	
	public double calculateStrain(double adcValue, double gain) {
		//final double vOffset = 1.5594482421875;
		//final double gainTotal = 825;
		//final double vOffset = offset;
		final double gainTotal = gain;
			
		double b = (adcValue - vZero)/(gainTotal*vExcitation);
		double a = b + (rG/(rG+r1));
		
		double strain = ((a*r1)/((1-a)*rG) - 1)*1000000/gageFactor;
		return strain;
	} 
	
	public boolean  zeroCalibration(double vOut) {
		if((vOut<(vZero-EPSILON)) || (vOut>(vZero+EPSILON))) {
			return false;	
		}	
		return true;
	}
    
    public double setActualGain(double adcValue, double actualStrain) {
    	double cActual=((actualStrain*gageFactor)/1000000)+1;
    	double aActual=cActual*rG/(r1+(cActual*rG));
    	double bActual=aActual-(rG/(r1+rG));
    	double actualGain=((adcValue-vZero)/(vExcitation*bActual));
    	return actualGain;		
    }
    
    //public static void main(String[] argv) throws IOException {
	public void setAction(String string_val,double set_val,String file_name) {
	String cmd;
	byte sequenceNo = 0;
	boolean read_log = false;
/***
	if (argv.length < 1) {
	    usage();
	    System.exit(-1);
	}
	cmd = argv[0];
	int time = Integer.parseInt(argv[1]);
	float val = Float.parseFloat(argv[2]);
	//int val = Integer.parseInt(argv[2]);
****/
	cmd=string_val;
	double val=set_val;
	int time=0;
	ADCMsg packet = new ADCMsg(); 

//	sequenceNo = restoreSequenceNo();
	sequenceNo = 0;
	packet.set_seqno(sequenceNo);
	packet.set_hop_count((short)0);
	packet.set_source(src_id);
	packet.set_time((short)0);
	packet.set_setting_val((short)0);

	if (cmd.equals("read_adc")) {
	    packet.set_action(READ_ADC);
	    packet.set_time((short)set_val);
	    packet.set_setting_val((int)1);
		gain_entry = 825.0;
		offset_entry = 1.1;
	    logfile=file_name;
	    read_log=true;
	    CALIBRATION=false;
	    try {
			   out = new FileOutputStream(logfile,true);
 				// Connect print stream to the output stream
				ps = new PrintStream( out );
		    	ps.println("Next Set of Readings ***********");
        		ps.close();
    		} catch (Exception e){
    			e.printStackTrace();
       		}	
	} else if (cmd.equals("set_power")) {
	    packet.set_action(SET_PWR);
	    packet.set_time((short)time);
	    //read_log=true;
	} else if (cmd.equals("prog_pga")) {
	    packet.set_action(PRG_PGA); 
	    //read_log=true;
	} else if (cmd.equals("set_gain")) {
	    packet.set_action(SET_GAIN);
	    packet.set_setting_val((int)val);
	    try {
	    	 if(src_id==2)
	    	 gainOut = new FileOutputStream("actualGain2.txt");
	    	 else
	    	 gainOut = new FileOutputStream("actualGain3.txt");
 		     ps1 = new PrintStream(gainOut);
			 ps1.println(""+val);
        	 ps1.close();
    		}		
    		catch (Exception e){
    		e.printStackTrace();
       		}	
	    //read_log=true;
	} else if (cmd.equals("set_offset")) {
	    packet.set_action(SET_OFFSET);
	    packet.set_setting_val((int)(val*10000));
	    //read_log=true;
	} else if (cmd.equals("read_reg")) {
	    packet.set_action(READ_REG);
	    packet.set_time((short)time);
	    read_log=true;
	} else if (cmd.equals("read_gain")) {
	    packet.set_action(READ_GAIN);
	    packet.set_time((short)time);
	    read_log=true;
 	    //packet.set_time((short)2000);
	} else if (cmd.equals("read_offset")) {
	    packet.set_action(READ_OFFSET);
	    packet.set_time((short)time);
	    read_log=true;
	    //packet.set_time((short)2000);
	} else if (cmd.equals("read_temp")) {
	    packet.set_action(READ_TEMP);
	    packet.set_time((short)time);
	    read_log=true;
	} else if (cmd.equals("mote_reset")) {
		packet.set_action(MOTE_RST);
		packet.set_time((short)0);
	} else if (cmd.equals("timer_stop")) {
		packet.set_action(TIMER_STOP);
		packet.set_time((short)0); 
	} else if (cmd.equals("zero_calibration")) {
		packet.set_action(READ_ADC);
	    packet.set_time((short)0);
	    read_log=true;	
	    CALIBRATION=true;
	} else if (cmd.equals("span_calibration")) {
		packet.set_action(READ_ADC);
	    packet.set_time((short)0);
	    read_log=true;	
	    SPAN_CALIBRATION=true;
	}
	else {
	    //usage();
	    System.exit(-1);
	}
        
	try {
	    System.err.print("Sending payload: ");
	  		
	    for (int i = 0; i < packet.dataLength(); i++) {
		System.err.print(Integer.toHexString(packet.dataGet()[i] & 0xff)+ " ");
	    }
	    System.err.println();

	    mote.send(TOS_BCAST_ADDR, packet);
	    // Need to wait for a read_log message to come back
    	
    	/*if (read_log) {
		synchronized (bc) {
		    if (bc.read_log_done == false) {
			System.err.println("Waiting for response to read_log...");
			//for(int i=0;i<100000;i++) {
			bc.wait(100000);
				}
		    if (bc.read_log_done == false) {
			System.err.println("Warning: Timed out waiting for response to read_log command!");
			    	}
				}
	    	}
	    */	
		} catch(Exception e) {
	    e.printStackTrace();
		}
	    saveSequenceNo(sequenceNo+1);
	   // if(!read_log)
	   // System.exit(0);
    		
	}//end of setAction
	

 	public void messageReceived(int dest_addr, Message m) {
	Volt_Msg lm = (Volt_Msg) m;
	char[] reg=new char[2];
	int adc_val;
	double adcOutput=0.0, finalStrain=0.0;
	String input_from_term=null;
	double strainActual=0.0;
	double gainActual=0.0,diff=0.0;
	double wait_time=0.0,newtime=0.0;
//	System.err.println("Received log message: "+lm);

//	System.err.print("Log values: ");
	switch(lm.get_type()) {
		case 1:
		    adc_val = lm.get_return_val();
	    	adcOutput=(adc_val*2.5)/4096;
	    		if(CALIBRATION)	{
	    		//	System.out.println("Setting1 with "+adcOutput);
	    			ADCMsg newPacket = new ADCMsg();
	    			newPacket.set_seqno((byte)0);
					newPacket.set_hop_count((short)0);
					newPacket.set_source(src_id);
					newPacket.set_time((short)0);
					newPacket.set_setting_val((short)0);
	    				if(!zeroCalibration(adcOutput)) {
	    					if(adcOutput>=vZero) {
									diff=adcOutput-vZero;
									setOffset=setOffset-diff;
									}
								else {
									diff=vZero-adcOutput;
									setOffset=setOffset+diff;
									}
	    					newPacket.set_action(SET_OFFSET);
	    					newPacket.set_setting_val((int)(setOffset*10000));
	    					System.out.println("Setting new offset to   "+setOffset);
	    					System.err.print("Sending payload: ");
	  		
	    					for (int i = 0; i < newPacket.dataLength(); i++) {
							System.err.print(Integer.toHexString(newPacket.dataGet()[i] & 0xff)+ " ");
	    					}
	    					System.err.println();
	    					try {
	    					mote.send(TOS_BCAST_ADDR, newPacket);
	    					} catch(Exception e) {
	    					e.printStackTrace();
							}	
							//wait_time=date.getTime();
							//while(newtime<wait_time+10) {
							//newtime=date.getTime();
						//	}
							wait_time=System.currentTimeMillis();
							System.out.println("init time is "+wait_time);
							while(newtime<wait_time+5000) {
							newtime=System.currentTimeMillis();
							}
	    					newPacket.set_action(READ_ADC);
	    					newPacket.set_time((short)0);
	    					newPacket.set_setting_val((int)(0.0));
	    					for (int i = 0; i < newPacket.dataLength(); i++) {
							System.err.print(Integer.toHexString(newPacket.dataGet()[i] & 0xff)+ " ");
	    					}
	    					System.err.println();	
	    					try {
	    					mote.send(TOS_BCAST_ADDR, newPacket);
	    					} catch(Exception e) {
	    					e.printStackTrace();
							}	
	    				}	
	    				else {
	    					System.out.println("Setting final offset");
	    					CALIBRATION=false;
	    				//	System.exit(0);
	    					}
	    					
	    			}
	    		  else if(SPAN_CALIBRATION) {
	    			 strainActual=Double.parseDouble(display.enter_strain.getText());
	    			 //System.out.println("Actual adc: "+adcOutput);
	    			 gainActual=setActualGain(adcOutput,strainActual);
	    			 System.out.println("Actual gain: "+gainActual);
	    			 SPAN_CALIBRATION=false;
	    				try {
	    					if(src_id==2)
	    				gainOut = new FileOutputStream("actualGain2.txt");
	    				else
	    				gainOut = new FileOutputStream("actualGain3.txt");
 					     ps1 = new PrintStream(gainOut);
		    			 ps1.println(""+gainActual);
        				 ps1.close();
    					}		
    					catch (Exception e){
    					e.printStackTrace();
       					}	
	    			 //System.exit(0);
	    		}
	    	//System.out.println("reading adc");
	    	try {
	    		if(lm.get_sourceaddr()==2)
       				inputFile=new BufferedReader(new InputStreamReader(new FileInputStream("actualGain2.txt")));
       			else
       				inputFile=new BufferedReader(new InputStreamReader(new FileInputStream("actualGain3.txt")));
       			gainActual=Double.parseDouble(inputFile.readLine());
   				
	    	finalStrain=calculateStrain(adcOutput,gainActual);
	    	//no averaging is taking place
	    	/*if(counter==20) {
			average_adc=total_adc/counter;
			average_uE=total_uE/counter; 
			counter=0;
			total_adc=0.0;average_adc=0.0;total_uE=0.0;average_uE=0.0;
	    	//System.out.println("Voltage: "+adcOutput +" Strain in uE: "+finalStrain+" Gain: "+gain_entry +" Offset: "+offset_entry);*/
			
			    out = new FileOutputStream(logfile,true);
 				// Connect print stream to the output stream
				ps = new PrintStream( out );
		    	ps.println("Voltage: "+adcOutput +" Strain in uE: "+finalStrain+" Gain: "+gain_entry +" Offset: "+offset_entry);
		    	if(lm.get_sourceaddr()==2)
        		display.display1.setText("Voltage: "+adcOutput +"          "+"Strain in uE: "+finalStrain);
        		else
        		display.display2.setText("Voltage: "+adcOutput +"          "+"Strain in uE: "+finalStrain);
        		//display.display.setBackground(Color.RED);
        		ps.close();
    			}		
    			catch (Exception e){
    			e.printStackTrace();
       			}	
				System.err.println("");
			/*} else {
			counter++;
			total_adc+=adcOutput;
			total_uE+=finalStrain;
			}*/	
        break;
	    
	    case 6:
	    	for (int i = 0; i < lm.numElements_log(); i++) {
	    		if(i<2)
	    		reg[i]=(char)lm.getElement_log(i);
	    	}
	    	System.out.println("Reg val is "+(char)(reg[0])+" "+Integer.toString(reg[1]));	
			break;
			
		case 7:
	    	System.out.println("Gain val:"+(float)lm.get_return_val());
	    	System.exit(1);
	    	break;
	    
	    case 8:	
	    	System.out.println("Offset val:"+(float)lm.get_return_val()/10000);
	    	System.exit(1);
			break;
	
	//synchronized (this) {
	 //   read_log_done = true;
	  //  this.notifyAll();
			}//switch
		}
    }	

