/* Author: Manjunath M Prabhu
 *
 * The newPanel class is the GUI for sending the command packet to 
 * 1. Program the PGA
 * 2. Set Gain and Offset
 * 3. Calibrate the guage
 * 4. Log the strain values
 */
package net.tinyos.tools;

import net.tinyos.util.*;
import net.tinyos.message.*;
import java.io.*;
import java.util.*;
import java.lang.*;
import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.NumberFormatter;
import java.text.ParseException;
import java.text.DecimalFormat;

public class newPanel extends Panel implements ActionListener {	


    JButton moteReset = new JButton("MOTE RESET");
    JButton progPGA = new JButton("PROGRAM PGA");
    JButton setGain = new JButton("SET GAIN");
    JButton setOffset = new JButton("SET OFFSET");
    JButton zeroCalib = new JButton("ZERO CALIBRATION");
    JButton spanCalib = new JButton("SPAN CALIBRATION");
    JButton readADC = new JButton("READ STRAIN");
    JButton timerStop =new JButton("TIMER STOP");
    
    //Not being used currently
    JButton readReg = new JButton("READ REGISTER");
    JButton readGain = new JButton("READ GAIN");
    JButton readOffset = new JButton("READ OFFSET");
    
    JFrame mainFrame;  
    public JFormattedTextField enter_gain,enter_offset, enter_strain, enter_timer, sourceID;
    JLabel source_id=new JLabel("Enter the Source ID");
    public JTextField display1,display2,enter_filename;
        
	cmds bg_prg=new cmds(this);
    public Panel controlPanel,readADCPanel,displayPanel,upperPanel;

	public newPanel() {
			mainFrame = new JFrame("Strain Readings");
			mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			mainFrame.setSize(new Dimension(900,600));
			upperPanel = new Panel();
			controlPanel = new Panel();
			readADCPanel = new Panel();
    		displayPanel = new Panel();
    		add("West",upperPanel);
			add("North",controlPanel);	
			add("Center",readADCPanel);	
			add("South",displayPanel);
			init();
			mainFrame.getContentPane().add(upperPanel,BorderLayout.WEST);
			mainFrame.getContentPane().add(controlPanel,BorderLayout.NORTH);
			mainFrame.getContentPane().add(readADCPanel,BorderLayout.CENTER);
			mainFrame.getContentPane().add(displayPanel,BorderLayout.SOUTH);
			mainFrame.show();
			mainFrame.pack();
			mainFrame.setVisible(true);	
    }
    
    public void init() {
    
    JPanel top=new JPanel();
    top.setLayout(new GridLayout(1,2));
	top.setPreferredSize(new Dimension(300,30));
	top.add(source_id);
	
		sourceID = new JFormattedTextField(new DecimalFormat("##0.0#"));
        sourceID.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);

        try {
            sourceID.setText("1");
            sourceID.commitEdit();
        } catch(ParseException e) {
            e.printStackTrace();
        }

	top.add(sourceID);
	
    
	JPanel left = new JPanel();
	left.setLayout(new GridLayout(3,1));
	left.setPreferredSize(new Dimension(150,100));
	left.add(moteReset);
	left.add(progPGA);
	left.add(zeroCalib);
	
	
	JPanel middle= new JPanel();
	middle.setLayout(new GridLayout(3,1));
	middle.setPreferredSize(new Dimension(150,100));
	middle.add(setGain);
	middle.add(setOffset);
	middle.add(spanCalib);
	

		enter_gain = new JFormattedTextField(new DecimalFormat("##0.0#"));
        enter_gain.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);

        try {
            enter_gain.setText("825.0");
            enter_gain.commitEdit();
        } catch(ParseException e) {
            e.printStackTrace();
        }

    	enter_offset = new JFormattedTextField(new DecimalFormat("##0.0#"));
        enter_offset.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);

        try {
            enter_offset.setText("1.1");
            enter_offset.commitEdit();
        } catch(ParseException e) {
            e.printStackTrace();
        }

		enter_strain = new JFormattedTextField(new DecimalFormat("##0.0#"));
        enter_strain.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);

        try {
            enter_strain.setText("1000");
            enter_strain.commitEdit();
        } catch(ParseException e) {
            e.printStackTrace();
        }
        
        display1= new JTextField(5000);
        try {
        	display1.getText();
        } catch(NullPointerException e) {
            //Shouldn't get here unless the setText value doesn't agree
            //with the format set above.
            e.printStackTrace();
        }
		display1.setFont(new Font("SansSerif",Font.BOLD,20));
		
		display2= new JTextField(5000);
        try {
        	display2.getText();
        } catch(NullPointerException e) {
            //Shouldn't get here unless the setText value doesn't agree
            //with the format set above.
            e.printStackTrace();
        }
		display2.setFont(new Font("SansSerif",Font.BOLD,20));
		
		enter_timer = new JFormattedTextField(new DecimalFormat("##0.0#"));
        enter_timer.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);

        try {
            enter_timer.setText("1.0");
            enter_timer.commitEdit();
        } catch(ParseException e) {
            e.printStackTrace();
        }
        
		enter_filename = new JTextField(30);
        try {
            enter_filename.setText("log.txt");
        } catch(Exception e) {
            e.printStackTrace();
        }
        
	JPanel right= new JPanel();
	right.setLayout(new GridLayout(3,1));
	right.setPreferredSize(new Dimension(100,100));
	right.add(enter_gain);
	right.add(enter_offset);
	right.add(enter_strain);
	 
	
	JPanel below=new JPanel();
	below.setLayout(new GridLayout(1,4));
	below.setPreferredSize(new Dimension(450,50));
	below.add(readADC);
	below.add(enter_timer);
	below.add(enter_filename);
	below.add(timerStop);
	
	
	JPanel text=new JPanel();
	text.setLayout(new GridLayout(2,1));
	text.setPreferredSize(new Dimension(900,200));
	text.add(display1);
	text.add(display2);
	//add(text);
	
	upperPanel.add(top);
	controlPanel.add(left);
	controlPanel.add(right);
	controlPanel.add(middle);
	readADCPanel.add(below);
	displayPanel.add(text);
	
	moteReset.addActionListener(this);
	progPGA.addActionListener(this);
	setGain.addActionListener(this);
	setOffset.addActionListener(this);
	zeroCalib.addActionListener(this);
	spanCalib.addActionListener(this);
	readADC.addActionListener(this);
	timerStop.addActionListener(this);	
	}

    public void actionPerformed(ActionEvent e) {
    String eventname =e.getActionCommand();	
	Object src = e.getSource();
	String cmd;
	double pass_val=0.0;
	bg_prg.src_id=(int)(Double.parseDouble(sourceID.getText()));
	String filename=(String)(enter_filename.getText());
	if (src == readADC) {
	 	cmd="read_adc";
	 	pass_val=(int)(Double.parseDouble(enter_timer.getText()));
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == progPGA) {
		cmd="prog_pga";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == setGain) {
		cmd="set_gain";
		pass_val= Double.parseDouble(enter_gain.getText());
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == setOffset) {
		cmd="set_offset";
		pass_val=Double.parseDouble(enter_offset.getText());
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == readReg) {
		cmd="read_reg";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == readGain) {
		cmd="read_gain";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == readOffset) {
		cmd="read_offset";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == moteReset) {
		cmd="mote_reset";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == timerStop) {
		cmd="timer_stop";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == zeroCalib) {
		cmd="zero_calibration";
	    bg_prg.setAction(cmd,pass_val,filename);
	} else if (src == spanCalib) {
		cmd="span_calibration";
	    bg_prg.setAction(cmd,pass_val,filename);
	}  
	}
	
//	public void sendMsg(String val) {
//		display.setText(""+val);
//	}

    // If specified as -1, then reset messages will only work properly
    // with the new TOSBase base station
    static int group_id = -1; 

	private static void createAndShowGUI() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		newPanel app=new newPanel();
	}
    
    public static void main(String[] args) throws IOException {
    javax.swing.SwingUtilities.invokeLater(new Runnable() 	{
		public void run() {
			createAndShowGUI();
		}
	});

   }//main ends here
}


